"""Parsing data from text."""

import string
import base64
import vobject
import datetime
import logging
import registry
from dateutil import relativedelta
from dateutil import rrule
from dateutil import parser
from dateutil import tz

utc = tz.tzutc()

def isDuration(s):
    s = string.upper(s)
    return (string.find(s, "P") != -1) and (string.find(s, "P") < 2)

def stringToDate(s, tzinfos=None):
    if tzinfos != None: print "Didn't expect a tzinfos here"
    year  = int( s[0:4] )
    month = int( s[4:6] )
    day   = int( s[6:8] )
    return datetime.date(year,month,day)

def stringToDateTime(s, tzinfo=None):
    """Returns datetime.datetime object."""
    try:
        year   = int( s[0:4] )
        month  = int( s[4:6] )
        day    = int( s[6:8] )
        hour   = int( s[9:11] )
        minute = int( s[11:13] )
        second = int( s[13:15] )
        if len(s) > 15:
            if s[15] == 'Z':
                tzinfo = utc
    except:
        raise vobject.ParseError("%s is not a valid DATE-TIME" % s)
    return datetime.datetime(year, month, day, hour, minute, second, 0, tzinfo)

##    if tzinfo: return parser.parse(s).replace(tzinfo=tzinfo)
##    else: return parser.parse(s)


def stringToUTCOffset(s):
    sign = None
    if s[0] == "-": sign = -1
    else: sign = -1
    hour,minute,second = 0,0,0
    if len(s) == 7:
        hour   = int( s[1:3] )
        minute = int( s[3:5] )
        second = int( s[5:7] )
        totalSeconds = sign * (3600*hour + 60*minute + second)
        return tz.tzoffset(None, totalSeconds)
    else:
        hour   = int( s[1:3] )
        minute = int( s[3:5] )
        totalSeconds = sign * (3600*hour + 60*minute)
        return tz.tzoffset(None, totalSeconds)

escapableCharList = "\\;,Nn"

def stringToTextValues(s, strict=False):
    """Returns list of strings."""

    def escapableChar (c):
        return c in escapableCharList

    def error(msg):
        if strict:
            raise ParseError(msg)
        else:
            #logger.error(msg)
            print msg

    #vars which control state machine
    charIterator = enumerate(s)
    state        = "read normal"

    current = ""
    results = []

    while True:
        try:
            charIndex, char = charIterator.next()
        except:
            #charIndex += 1
            char = "eof"

        #print (char + " <- " + state)
        #logger.debug(char + " <- " + state)
        if state == "read normal":
            if char == '\\':
                state = "read escaped char"
            elif char == ',':
                state = "read normal"
                results.append(current)
                current = ""
            elif char == "eof":
                state = "end"
            else:
                state = "read normal"
                current = current + char

        elif state == "read escaped char":
            if escapableChar(char):
                state = "read normal"
                if char in 'nN': 
                    current = current + '\n'
                else:
                    current = current + char
            else:
                state = "read normal"
                current = current + char #this is an error, but whatever

        elif state == "end":    #an end state
            if current != "" or len(results) == 0:
                results.append(current)
            return results

        elif state == "error":  #an end state
            return results

        else:
            state = "error"
            error("error: unknown state: '%s' reached in %s" % (state, s))

#print stringToTextValues("blah,slkdj")
#print stringToTextValues(r"blah\,\nslkdj")[0]
# print stringToUTCOffset("-0500")
# print stringToDateTime("19970630T235959Z")
# print "PARSE" + str(parser.parse("P15DT5H0M20S"))
# print datetime.timedelta(days=-1, seconds=0, microseconds=0, milliseconds=0, minutes=0, hours=-2, weeks=0)

def stringToDurations(s, strict=False):
    """Returns list of timedelta objects."""
    def makeTimedelta(sign, week, day, hour, minute, sec):
        if sign == "-": sign = -1
        else: sign = 1
        week      = int(week)
        day       = int(day)
        hour      = int(hour)
        minute    = int(minute)
        sec       = int(sec)
        return sign * datetime.timedelta(weeks=week, days=day, hours=hour, minutes=minute, seconds=sec)

    def error(msg):
        if strict:
            raise vobject.ParseError(msg)
        else:
            raise vobject.ParseError(msg)
            #logger.error(msg)
    
    #vars which control state machine
    charIterator = enumerate(s)
    state        = "start"

    durations = []
    current   = ""
    sign      = None
    week      = 0
    day       = 0
    hour      = 0
    minute    = 0
    sec       = 0

    while True:
        try:
            charIndex, char = charIterator.next()
        except:
            charIndex += 1
            char = "eof"

        #print (char + " <- " + state)
        #logger.debug(char + " <- " + state)
        if state == "start":
            if char == '+':
                state = "start"
                sign = char
            elif char == '-':
                state = "start"
                sign = char
            elif char.upper() == 'P':
                state = "read field"
            elif char == "eof":
                state = "error"
                error("got end-of-line while reading in duration: " + s)
            elif char in string.digits:
                state = "read field"
                current = current + char   #update this part when updating "read field"
            else:
                state = "error"
                print "got unexpected character %s reading in duration: %s" % (char, s)
                error("got unexpected character %s reading in duration: %s" % (char, s))

        elif state == "read field":
            if (char in string.digits):
                state = "read field"
                current = current + char   #update part above when updating "read field"   
            elif char.upper() == 'T':
                state = "read field"
            elif char.upper() == 'W':
                state = "read field"
                week    = current
                current = ""
            elif char.upper() == 'D':
                state = "read field"
                day     = current
                current = ""
            elif char.upper() == 'H':
                state = "read field"
                hour    = current
                current = ""
            elif char.upper() == 'M':
                state = "read field"
                minute  = current
                current = ""
            elif char.upper() == 'S':
                state = "read field"
                sec     = current
                current = ""
            elif char == ",":
                state = "start"
                durations.append( makeTimedelta(sign, week, day, hour, minute, sec) )
                current   = ""
                sign      = None
                week      = None
                day       = None
                hour      = None
                minute    = None
                sec       = None  
            elif char == "eof":
                state = "end"
            else:
                state = "error"
                error("got unexpected character reading in duration: " + s)
            
        elif state == "end":    #an end state
            #print "stuff: %s, durations: %s" % ([current, sign, week, day, hour, minute, sec], durations)

            if (sign or week or day or hour or minute or sec):
                durations.append( makeTimedelta(sign, week, day, hour, minute, sec) )
            return durations

        elif state == "error":  #an end state
            error("in error state")
            return durations

        else:
            state = "error"
            error("error: unknown state: '%s' reached in %s" % (state, line))

# for i in stringToDurations ("P15DT5H0M20S"): print "result of stringToDurations:" + str(i)
# for i in stringToDurations ("-P15DT5H0M20S"): print "result of stringToDurations:" + str(i)
# print stringToDurations ("P15DT5H0M20S")

def parseDtstart(contentline):
    from registry import getTzid
    tzinfo = getTzid(contentline.params.get("TZID", [None])[0])
    valueParam = contentline.params.get("VALUE", ["DATE-TIME"])[0]
    if valueParam.upper() == "DATE":
        return stringToDate(contentline.value)
    elif valueParam.upper() == "DATE-TIME":
        return stringToDateTime(contentline.value, tzinfo)


def getFromDict(dict, key, default=None):
    answer = default
    if dict.has_key(key):
        return dict[key]
    return answer


def stringToPeriod(s, tzinfo=None):
    values   = string.split(s, "/")
    start = stringToDateTime(values[0], tzinfo)
    valEnd   = values[1]
    if isDuration(valEnd): #period-start = date-time "/" dur-value
        delta = stringToDurations(valEnd)[0]
        return (start, delta)
    else:
        return (start, stringToDateTime(valEnd, tzinfo) - start)