"""Serializing data to text."""

import string
import base64
import math
import datetime
import vobject
import parsing

from dateutil import relativedelta
from dateutil import rrule
from dateutil import parser
from dateutil import tz

from vobject import ContentLine

## Utils

def numToDigits(num, places):
    """Helper, for converting numbers to textual digits."""
    s = str(num)
    if len(s) < places:
        return ("0" * (places - len(s))) + s
    elif len(s) > places:
        return s[len(s)-places: ]
    else:
        return s

def floor(x, y):
    i = math.floor(x/y)
    rem = x % y
    return i,rem

def uriToString(uri):
    """Returns a string URI."""
    return uri

def timedeltaToString(delta):
    """Convert timedelta to an rfc2445 DURATION."""
    if delta.days == 0: sign = 1
    else: sign = delta.days / abs(delta.days)
    delta = abs(delta)
    days = delta.days
    hours = delta.seconds / 3600
    minutes = (delta.seconds % 3600) / 60
    seconds = delta.seconds % 60
    out = ''
    if sign == -1: out = '-'
    out += 'P'
    if days: out += str(days) + 'D'
    if hours or minutes or seconds: out += 'T'
    elif not days: #Deal with zero duration
        out += '0S'
    if hours: out += str(hours) + 'H'
    if minutes: out += str(minutes) + 'M'
    if seconds: out += str(seconds) + 'S'
    return out

def testNumToDigits():
    assert( numToDigits(2, 3)       == '002' )
    assert( numToDigits(000002, 3)  == '002' )
    assert( numToDigits(2000002, 3) == '002' )
    assert( numToDigits(202, 3)     == '202' )

def dateToString(date):
    year  = numToDigits( date.year,  4 )
    month = numToDigits( date.month, 2 )
    day   = numToDigits( date.day,   2 )
    return year + month + day

def dateTimeToString(dateTime, preserveTZ=True):
    """Convert to UTC if tzinfo is set, unless preserveTZ.  Output string."""
    if dateTime.tzinfo and not preserveTZ:
        dateTime = dateTime.astimezone(tz.tzutc())
    if dateTime.tzinfo == tz.tzutc(): utcString = "Z"
    else: utcString = ""

    year  = numToDigits( dateTime.year,  4 )
    month = numToDigits( dateTime.month, 2 )
    day   = numToDigits( dateTime.day,   2 )
    hour  = numToDigits( dateTime.hour,  2 )
    mins  = numToDigits( dateTime.minute,  2 )
    secs  = numToDigits( dateTime.second,  2 )

    return year + month + day + "T" + hour + mins + secs + utcString

def deltaToOffset(delta):
    absDelta = abs(delta)
    hours = absDelta.seconds / 3600
    hoursString      = numToDigits(hours, 2)
    minutesString    = '00'
    if absDelta == delta:
        signString = "+"
    else:
        signString = "-"
    return signString + hoursString + minutesString

def textToString(s):
    result = ""
    for c in s:
        if c in parsing.escapableCharList:
            result += "\\" + c
        else: 
            result += c
    return result

def serializeAttachment (contentlineName, attachment):
    if isinstance(attachment, str):
        # encodestring does this shockingly retarded thing where always adds a newline
        return ContentLine(contentlineName,
                           {"VALUE":"BINARY", "ENCODING":"BASE64"},
                           base64.encodestring(attachment)[ :-1])
    else:
        # attachment is of type Attachment
        return ContentLine( contentlineName,
                            {"FMTTYPE" : attachment.format},
                            uriToString(attachment.url) )


def serializeDateTime(contentlineName, value):
    return ContentLine( contentlineName, {}, dateTimeToString(value) )


def serializeRdate (contentlineName, value):
    if isinstance(value, DateRdate):
        table = {"VALUE":"DATE"}
    elif isinstance(value, PeriodRdate):
        table = {"VALUE":"PERIOD"}
    else:
        table = {"VALUE":"DATE-TIME"}
    if value.tzid != None:
        table["TZID"] = value.tzid
    return ContentLine(contentlineName, table, string.join(value.values, ","))